package com.ruoyi.web.controller.oa;

import java.math.BigDecimal;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.stream.Collectors;
import javax.servlet.http.HttpServletResponse;

import com.alibaba.fastjson2.JSONArray;
import com.alibaba.fastjson2.JSONObject;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.oa.domain.CmcCarApproval;
import com.ruoyi.oa.domain.CmcProject;
import com.ruoyi.oa.service.ICmcCarApprovalService;
import com.ruoyi.system.service.ISysUserService;
import org.springframework.beans.factory.annotation.Autowired;

import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.oa.domain.CmcCar;
import com.ruoyi.oa.service.ICmcCarService;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.common.core.page.TableDataInfo;

/**
 * cmc车辆信息Controller
 *
 * @author cmc
 * @date 2024-02-23
 */
@RestController
@RequestMapping("/oa/car")
public class CmcCarController extends BaseController
{
    @Autowired
    private ICmcCarService cmcCarService;

    @Autowired
    private ICmcCarApprovalService cmcCarApprovalService;

    /**
     * 查询cmc车辆信息列表
     */
    @GetMapping("/list")
    public TableDataInfo list(CmcCar cmcCar)
    {
        startPage();
        List<CmcCar> list = cmcCarService.selectCmcCarList(cmcCar);
        return getDataTable(list);
    }

    /**
     * 导出cmc车辆信息列表
     */
    @Log(title = "cmc车辆信息", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, CmcCar cmcCar)
    {
        List<CmcCar> list = cmcCarService.selectCmcCarList(cmcCar);
        ExcelUtil<CmcCar> util = new ExcelUtil<CmcCar>(CmcCar.class);
        util.exportExcel(response, list, "cmc车辆信息数据");
    }

    /**
     * 获取cmc车辆信息详细信息
     */
    @GetMapping(value = "/{carId}")
    public AjaxResult getInfo(@PathVariable("carId") Integer carId)
    {
        return success(cmcCarService.selectCmcCarByCarId(carId));
    }

    /**
     * 获取设备统计信息
     */
    @GetMapping(value = "/statistic")
    public AjaxResult getCarStatistic(CmcCarApproval cmcCarApproval) throws ParseException
    {
        JSONObject jsonObject = new JSONObject();
        JSONArray yearArray = new JSONArray();
        JSONObject yearObject = new JSONObject();
        JSONArray statusArray = new JSONArray();
        JSONObject statusObject = new JSONObject();
        JSONArray approvalArray = new JSONArray();
        JSONObject approvalObject = new JSONObject();
        CmcCar cmcCar = new CmcCar();
        cmcCar.setStatus("0");
        statusObject.put("已派出", cmcCarService.selectCmcCarList(cmcCar).size());
        cmcCar.setStatus("1");
        statusObject.put("可使用", cmcCarService.selectCmcCarList(cmcCar).size());
        cmcCar.setStatus("2");
        statusObject.put("维修中", cmcCarService.selectCmcCarList(cmcCar).size());
        cmcCar.setStatus("3");
        statusObject.put("已还车", cmcCarService.selectCmcCarList(cmcCar).size());
        cmcCar.setStatus("4");
        statusObject.put("已报废", cmcCarService.selectCmcCarList(cmcCar).size());
        //整体
        if (cmcCarApproval.getApplyDate() == null) {
            //每年派车次数
            for (int i = 2019; i <= Calendar.getInstance().get(Calendar.YEAR); i++) {
                //总数
                cmcCarApproval.setApplyDate(new SimpleDateFormat("yyyy").parse(String.valueOf(i)));
                List<CmcCarApproval> yearList = cmcCarApprovalService.selectCmcCarApprovalList(cmcCarApproval);
                yearObject.put(String.valueOf(i), yearList.size());
            }
            cmcCarApproval.setApplyDate(null);
        }
        //年度
        else {
            //输入年份派车次数
            yearObject.put(new SimpleDateFormat("yyyy").format(cmcCarApproval.getApplyDate()), cmcCarApprovalService.selectCmcCarApprovalList(cmcCarApproval).size());
        }
        getCarApprovalStatistic(cmcCarApproval, approvalObject);
        yearArray.add(yearObject);
        statusArray.add(statusObject);
        approvalArray.add(approvalObject);
        jsonObject.put("year", yearArray);
        jsonObject.put("status",statusArray);
        jsonObject.put("approval",approvalArray);
        return success(jsonObject);
    }

    /**
     * 新增cmc车辆信息
     */
    @Log(title = "cmc车辆信息", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody CmcCar cmcCar)
    {
        cmcCar.setStatus("1");
        if (cmcCar.getAcquisitionTime() != null && cmcCar.getCost() != null && cmcCar.getDayCost() == null) {
            // 年数总和法折旧
            int acquisitionYear = Integer.parseInt(new SimpleDateFormat("yyyy").format(cmcCar.getAcquisitionTime()));
            int life = cmcCar.getExpectLife() == null ? 0 : cmcCar.getExpectLife();
            int total = 0;
            for (int i = 0; i < life + 1; i++)
                total += i;
            if (total > 0) {
                int currentYear = Integer.parseInt(new SimpleDateFormat("yyyy").format(DateUtils.getNowDate()));
                int num = life - (currentYear - acquisitionYear) + 1;
                BigDecimal estimateCost = new BigDecimal(num  * 0.95 / total / 365).multiply(cmcCar.getCost());
                cmcCar.setDayCost(estimateCost);
            }
        }
        return toAjax(cmcCarService.insertCmcCar(cmcCar));
    }

    /**
     * 修改cmc车辆信息
     */
    @Log(title = "cmc车辆信息", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody CmcCar cmcCar)
    {
        if (cmcCar.getAcquisitionTime() != null && cmcCar.getCost() != null && cmcCar.getDayCost() == null) {
            // 年数总和法折旧
            int acquisitionYear = Integer.parseInt(new SimpleDateFormat("yyyy").format(cmcCar.getAcquisitionTime()));
            int life = cmcCar.getExpectLife() == null ? 0 : cmcCar.getExpectLife();
            int total = 0;
            for (int i = 0; i < life + 1; i++)
                total += i;
            if (total > 0) {
                int currentYear = Integer.parseInt(new SimpleDateFormat("yyyy").format(DateUtils.getNowDate()));
                int num = life - (currentYear - acquisitionYear) + 1;
                BigDecimal estimateCost = new BigDecimal(num  * 0.95 / total / 365).multiply(cmcCar.getCost());
                cmcCar.setDayCost(estimateCost);
            }
        }
        return toAjax(cmcCarService.updateCmcCar(cmcCar));
    }

    /**
     * 删除cmc车辆信息
     */
    @Log(title = "cmc车辆信息", businessType = BusinessType.DELETE)
	@DeleteMapping("/{carIds}")
    public AjaxResult remove(@PathVariable Long[] carIds)
    {
        return success(cmcCarService.deleteCmcCarByCarIds(carIds));
    }

    //各车辆派出次数
    public void getCarApprovalStatistic(CmcCarApproval cmcCarApproval, JSONObject approvalObject) {
        //类型太多，循环效率不高-直接用Java特性进行分组统计
        Map<Object, Long> typeGroup = cmcCarApprovalService.selectCmcCarApprovalList(cmcCarApproval).stream().collect(Collectors.groupingBy(item -> Optional.ofNullable(item.getCars()), Collectors.counting()));
        List<Map.Entry<Object, Long>> typeList = new ArrayList<>(typeGroup.entrySet());
        typeList.sort(Map.Entry.comparingByValue());
        typeList.forEach((item) -> {
            if (item.getKey() != Optional.empty())
                approvalObject.put(item.toString().substring(item.toString().indexOf("[") + 1, item.toString().indexOf("]")), item.getValue());
        });
    }

}
