package com.ruoyi.web.controller.oa;

import java.math.BigDecimal;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.stream.Collectors;
import javax.servlet.http.HttpServletResponse;

import com.alibaba.fastjson2.JSONArray;
import com.alibaba.fastjson2.JSONObject;
import com.ruoyi.oa.domain.CmcPartyA;
import com.ruoyi.oa.domain.CmcProject;
import com.ruoyi.oa.service.ICmcPartyAService;
import org.apache.poi.hpsf.Decimal;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.oa.domain.CmcContract;
import com.ruoyi.oa.service.ICmcContractService;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.common.core.page.TableDataInfo;

/**
 * cmc合同评审Controller
 * 
 * @author cmc
 * @date 2024-03-15
 */
@RestController
@RequestMapping("/oa/contract")
public class CmcContractController extends BaseController
{
    @Autowired
    private ICmcContractService cmcContractService;

    /**
     * 查询cmc合同评审列表
     */
    @GetMapping("/list")
    public TableDataInfo list(CmcContract cmcContract)
    {
        startPage();
        List<CmcContract> list = cmcContractService.selectCmcContractList(cmcContract);
        return getDataTable(list);
    }

    /**
     * 获取cmc承接合同统计信息
     */
    @GetMapping("/statistic")
    public AjaxResult getContractStatistic(CmcContract cmcContract) throws ParseException {
        JSONObject jsonObject = new JSONObject();
        JSONArray yearArray = new JSONArray();
        JSONObject yearObject = new JSONObject();
        JSONArray sourceArray = new JSONArray();
        JSONObject sourceObject = new JSONObject();
        JSONArray amountArray = new JSONArray();
        JSONObject amountObject = new JSONObject();
        JSONArray cwAmountArray = new JSONArray();
        JSONObject cwAmountObject = new JSONObject();
        //每年合同数量及金额
        if (cmcContract.getSignDate() == null) {
            for (int i = 2019; i <= Calendar.getInstance().get(Calendar.YEAR); i++) {
                cmcContract.setSignDate(new SimpleDateFormat("yyyy").parse(String.valueOf(i)));
                List<CmcContract> yearList = cmcContractService.selectCmcContractList(cmcContract);
                yearObject.put(String.valueOf(i), yearList.size());
                BigDecimal amount = new BigDecimal(0);
                for (CmcContract contract : yearList) {
                    amount = amount.add(contract.getAmount());
                }
                amountObject.put(String.valueOf(i), amount);

            }
            cmcContract.setSignDate(null);
        }
        else {
            //输入年份合同数量及金额
            yearObject.put(new SimpleDateFormat("yyyy").format(cmcContract.getSignDate()), cmcContractService.selectCmcContractList(cmcContract).size());
            BigDecimal amount = new BigDecimal(0);
            for (CmcContract contract : cmcContractService.selectCmcContractList(cmcContract)) {
                amount = amount.add(contract.getAmount());
            }
            amountObject.put(new SimpleDateFormat("yyyy").format(cmcContract.getSignDate()), amount);
        }
        getProjectSourceStatistic(cmcContract, sourceObject, cwAmountObject);
        yearArray.add(yearObject);
        sourceArray.add(sourceObject);
        amountArray.add(amountObject);
        cwAmountArray.add(cwAmountObject);
        jsonObject.put("year", yearArray);
        jsonObject.put("source", sourceArray);
        jsonObject.put("amount", amountArray);
        jsonObject.put("cwAmount", cwAmountArray);
        return success(jsonObject);
    }

    /**
     * 导出cmc合同评审列表
     */
    @Log(title = "cmc合同评审", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, CmcContract cmcContract)
    {
        List<CmcContract> list = cmcContractService.selectCmcContractList(cmcContract);
        ExcelUtil<CmcContract> util = new ExcelUtil<CmcContract>(CmcContract.class);
        util.exportExcel(response, list, "cmc合同评审数据");
    }

    /**
     * 获取cmc合同评审详细信息
     */
    @GetMapping(value = "/{contractId}")
    public AjaxResult getInfo(@PathVariable("contractId") String contractId)
    {
        return success(cmcContractService.selectCmcContractByContractId(contractId));
    }

    /**
     * 新增cmc合同评审
     */
    @Log(title = "cmc合同评审", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody CmcContract cmcContract)
    {
        return toAjax(cmcContractService.insertCmcContract(cmcContract));
    }

    /**
     * 修改cmc合同评审
     */
    @Log(title = "cmc合同评审", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody CmcContract cmcContract)
    {
        if (cmcContract.getManagerComment() != null && cmcContract.getGmComment() == null) {
            cmcContract.setManagerUserId(getLoginUser().getUserId());
        }
        if (cmcContract.getGmComment() != null && cmcContract.getSignDate() == null && getLoginUser().getUserId() != 22) {
            cmcContract.setGmUserId(getLoginUser().getUserId());
        }
        return toAjax(cmcContractService.updateCmcContract(cmcContract));
    }

    /**
     * 删除cmc合同评审
     */
    @Log(title = "cmc合同评审", businessType = BusinessType.DELETE)
	@DeleteMapping("/{contractIds}")
    public AjaxResult remove(@PathVariable String[] contractIds)
    {
        return success(cmcContractService.deleteCmcContractByContractIds(contractIds));
    }

    //院内外项目数量及金额
    public void getProjectSourceStatistic(CmcContract cmcContract, JSONObject sourceObject, JSONObject cwAmountObject) {
        cmcContract.setProjectSource("1");
        List<CmcContract> sourceList = cmcContractService.selectCmcContractList(cmcContract);
        BigDecimal cAmount = new BigDecimal(0);
        BigDecimal wAmount = new BigDecimal(0);
        for (CmcContract contract : sourceList) {
            wAmount = wAmount.add(contract.getAmount());
        }
        sourceObject.put("院外", sourceList.size());
        cwAmountObject.put("院外", wAmount);
        cmcContract.setProjectSource("0");
        sourceList = cmcContractService.selectCmcContractList(cmcContract);
        for (CmcContract contract : sourceList) {
            cAmount = cAmount.add(contract.getAmount());
        }
        sourceObject.put("院内", sourceList.size());
        cwAmountObject.put("院内", cAmount);
        cmcContract.setProjectSource(null);
    }

}
