package com.ruoyi.web.controller.oa;

import java.io.IOException;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.stream.Collectors;
import javax.servlet.http.HttpServletResponse;

import com.alibaba.fastjson2.JSONArray;
import com.alibaba.fastjson2.JSONObject;
import com.ruoyi.common.core.domain.entity.SysUser;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.oa.domain.CmcArchive;
import com.ruoyi.oa.domain.CmcManageDept;
import com.ruoyi.oa.domain.CmcSettle;
import com.ruoyi.oa.service.*;
import com.ruoyi.system.service.ISysDeptService;
import com.ruoyi.system.service.ISysUserService;
import com.ruoyi.web.controller.qyweixin.MessageController;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.oa.domain.CmcProject;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.common.core.page.TableDataInfo;

/**
 * cmc项目Controller
 * 
 * @author cmc
 * @date 2024-01-30
 */
@RestController
@RequestMapping("/oa/project")
public class CmcProjectController extends BaseController
{
    @Autowired
    private ICmcProjectService cmcProjectService;

    @Autowired
    private ICmcPartyAService cmcPartyAService;

    @Autowired
    private ICmcProjectWorkService cmcProjectWorkService;

    @Autowired
    private ISysDeptService deptService;

    @Autowired
    private ISysUserService userService;

    @Autowired
    private ICmcManageDeptService cmcManageDeptService;

    @Autowired
    private ICmcArchiveService cmcArchiveService;

    @Autowired
    private ICmcSettleService cmcSettleService;

    /**
     * 查询cmc项目列表
     */
    @GetMapping("/list")
    public TableDataInfo list(CmcProject cmcProject)
    {
        startPage();
        List<CmcProject> list = new ArrayList<>();
        if (cmcProject.getUndertakingDept() != null && cmcProject.getUndertakingDept().equals("113"))
            list = cmcProjectService.selectCmcInvestProjectList(cmcProject);
        else
            list = cmcProjectService.selectCmcProjectList(cmcProject);
        for (CmcProject project : list) {
            project.setUndertakingDeptName(deptIdToName(project));
            project.setQualityInspectorName(inspectorIdToName(project));
        }
        return getDataTable(list);
    }

    /**
     * 导出cmc项目列表
     */
    @Log(title = "cmc项目", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, CmcProject cmcProject)
    {
        List<CmcProject> list = new ArrayList<>();
        if (cmcProject.getUndertakingDept() != null && cmcProject.getUndertakingDept().equals("113"))
            list = cmcProjectService.selectCmcInvestProjectList(cmcProject);
        else
            list = cmcProjectService.selectCmcProjectList(cmcProject);
        for (CmcProject project : list) {
            project.setUndertakingDeptName(deptIdToName(project));
            project.setQualityInspectorName(inspectorIdToName(project));
        }
        ExcelUtil<CmcProject> util = new ExcelUtil<CmcProject>(CmcProject.class);
        util.exportExcel(response, list, "cmc项目数据");
    }

    /**
     * 获取cmc项目详细信息
     */
    @GetMapping(value = "/{projectId}")
    public AjaxResult getInfo(@PathVariable("projectId") String projectId)
    {
        CmcProject cmcProject = cmcProjectService.selectCmcProjectByProjectId(projectId);
        cmcProject.setUndertakingDeptName(deptIdToName(cmcProject));
        String qualityInspector = cmcProject.getQualityInspector();
        StringBuilder qualityInspectorName = new StringBuilder();
        if (qualityInspector != null && !qualityInspector.equals("") && qualityInspector.split(",").length > 0) {
            for (String inspector : qualityInspector.split(","))
                qualityInspectorName.append(userService.selectUserById(Long.parseLong(inspector)).getNickName()).append(",");
            cmcProject.setQualityInspectorName(qualityInspectorName.substring(0, qualityInspectorName.length() - 1));
        }
        return success(cmcProject);
    }

    /**
     * 项目自动编号
     */
    @GetMapping("/projectNumber")
    public AjaxResult getProjectNumberByYear(CmcProject cmcProject)
    {
        List<CmcProject> list = cmcProjectService.selectCmcAllProjectList(cmcProject);
        String number = list.get(0).getProjectNumber().substring(5);
        String projectNumber = cmcProject.getProjectNumber() + (Integer.parseInt(number) + 1);
        return success(projectNumber);
    }

    /**
     * 项目类型列表
     */
    @GetMapping("/projectType")
    public AjaxResult getProjectType()
    {
        List<String> typeList = new ArrayList<>();
        Map<Object, List<CmcProject>> projectType = cmcProjectService.selectCmcAllProjectList(new CmcProject()).stream().collect(Collectors.groupingBy(item -> Optional.ofNullable(item.getProjectType())));
        projectType.forEach((k, v) -> {
            if (k != Optional.empty())
                typeList.add(k.toString().substring(k.toString().indexOf("[") + 1, k.toString().indexOf("]")));
        });
        Collections.sort(typeList);
        return success(typeList);
    }

    /**
     * 获取cmc项目统计信息
     */
    @GetMapping("/statistic")
    public AjaxResult getProjectStatistic(CmcProject cmcProject) throws ParseException {
        JSONObject jsonObject = new JSONObject();
        JSONArray yearArray = new JSONArray();
        JSONObject yearObject = new JSONObject();
        JSONArray sourceArray = new JSONArray();
        JSONObject sourceObject = new JSONObject();
        JSONArray typeArray = new JSONArray();
        JSONObject typeObject = new JSONObject();
        JSONArray deptArray = new JSONArray();
        JSONObject deptObject = new JSONObject();
        JSONArray completeArray = new JSONArray();
        JSONObject completeObject = new JSONObject();
        JSONArray archiveArray = new JSONArray();
        JSONObject archiveObject = new JSONObject();
        JSONArray settleArray = new JSONArray();
        JSONObject settleObject = new JSONObject();
        //整体
        if (cmcProject.getProjectNumber() == null) {
            //每年项目数量
            for (int i = 2019; i <= Calendar.getInstance().get(Calendar.YEAR); i++) {
                //总数
                cmcProject.setProjectNumber(String.valueOf(i));
                List<CmcProject> yearList = cmcProjectService.selectCmcAllProjectList(cmcProject);
                yearObject.put(String.valueOf(i), yearList.size());
                //已完成
                cmcProject.setIsComplete("1");
                List<CmcProject> completeList = cmcProjectService.selectCmcAllProjectList(cmcProject);
                completeObject.put(cmcProject.getProjectNumber(), completeList.size());
                cmcProject.setIsComplete(null);
                //已归档
                CmcArchive cmcArchive = new CmcArchive();
                cmcArchive.setProjectNumber(String.valueOf(i));
                cmcArchive.setArchiveTime(new SimpleDateFormat("yyyy").parse("2000-01-01"));
                List<CmcArchive> archiveList = cmcArchiveService.selectCmcArchiveList(cmcArchive);
                archiveObject.put(String.valueOf(i), archiveList.size());
                //已结算
                CmcSettle cmcSettle = new CmcSettle();
                cmcSettle.setProjectNumber(String.valueOf(i));
                cmcSettle.setGmTime(new SimpleDateFormat("yyyy").parse("2000-01-01"));
                List<CmcSettle> settleList = cmcSettleService.selectCmcSettleList(cmcSettle);
                settleObject.put(String.valueOf(i), settleList.size());
            }
            cmcProject.setProjectNumber(null);

        }
        //年度
        else {
            //输入年份项目数量
            List<CmcProject> yearList = cmcProjectService.selectCmcAllProjectList(cmcProject);
            //总数
            yearObject.put(cmcProject.getProjectNumber(),  yearList.size());
            //已完成
            cmcProject.setIsComplete("1");
            List<CmcProject> completeList = cmcProjectService.selectCmcAllProjectList(cmcProject);
            completeObject.put(cmcProject.getProjectNumber(), completeList.size());
            cmcProject.setIsComplete(null);
            //已归档
            CmcArchive cmcArchive = new CmcArchive();
            cmcArchive.setProjectNumber(cmcProject.getProjectNumber());
            cmcArchive.setArchiveTime(new SimpleDateFormat("yyyy").parse("2000-01-01"));
            List<CmcArchive> archiveList = cmcArchiveService.selectCmcArchiveList(cmcArchive);
            archiveObject.put(cmcProject.getProjectNumber(), archiveList.size());
            //已结算
            CmcSettle cmcSettle = new CmcSettle();
            cmcSettle.setProjectNumber(cmcProject.getProjectNumber());
            cmcSettle.setGmTime(new SimpleDateFormat("yyyy").parse("2000-01-01"));
            List<CmcSettle> settleList = cmcSettleService.selectCmcSettleList(cmcSettle);
            settleObject.put(cmcProject.getProjectNumber(), settleList.size());

        }
        getProjectSourceStatistic(cmcProject, sourceObject);
        getProjectTypeStatistic(cmcProject, typeObject);
        getProjectDeptStatistic(cmcProject, deptObject);
        yearArray.add(yearObject);
        sourceArray.add(sourceObject);
        deptArray.add(deptObject);
        typeArray.add(typeObject);
        completeArray.add(completeObject);
        archiveArray.add(archiveObject);
        settleArray.add(settleObject);
        jsonObject.put("year", yearArray);
        jsonObject.put("source", sourceArray);
        jsonObject.put("dept", deptArray);
        jsonObject.put("type", typeArray);
        jsonObject.put("complete", completeArray);
        jsonObject.put("archive", archiveArray);
        jsonObject.put("settle", settleArray);
        return success(jsonObject);
    }

    /**
     * 新增cmc项目
     */
    @Log(title = "cmc项目", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody CmcProject cmcProject) throws IOException {
        cmcProject.setRegisterTime(new Date());
        cmcProject.setIsFinished("0");
        if (cmcProject.getUndertakingDept().contains(",")) {
            cmcProject.setLeadDept(cmcProject.getUndertakingDept().substring(0,3));
            cmcProject.setJoinDept(cmcProject.getUndertakingDept().substring(4));
        }
        else
            cmcProject.setLeadDept(cmcProject.getUndertakingDept());
        return toAjax(cmcProjectService.insertCmcProject(cmcProject));
    }

    /**
     * 修改cmc项目
     */
    @Log(title = "cmc项目", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody CmcProject cmcProject)
    {
        return toAjax(cmcProjectService.updateCmcProject(cmcProject));
    }

    /**
     * 登记项目发送企业微信消息
     */
    @PostMapping("/sendMessage")
    public AjaxResult sendMessage(@RequestBody CmcProject cmcProject) throws IOException {
        String message = "经营发展部已发起项目登记：  \n>" +
                "登记人： <font color='info'>" + userService.selectUserById(cmcProject.getProjectRegistrant()).getNickName() + "</font>  \n>" +
                "项目编号：" + cmcProject.getProjectNumber() + "  \n>" +
                "项目名称：" + cmcProject.getProjectName() + "  \n>" +
                "  \n>" +
                "项目类型：<font color='comment'>" + cmcProject.getProjectType() + "</font>  \n>" +
                "项目来源：" + (cmcProject.getProjectSource().equals("0") ? "院内" : "院外") +"  \n>" +
                "甲方单位：" + cmcPartyAService.selectCmcPartyAByPartyAId(cmcProject.getPartyAId()).getPartyAName() + "  \n>" +
                "  \n>" +
                "承担部门：<font color='warning'>" + deptIdToName(cmcProject) + "</font>  \n>" +
                "请您确认承担部门，及时跟进项目流转  \n>" +
                "  \n>";
        if (cmcProject.getUndertakingDept() != null) {
            String deptString = cmcProject.getUndertakingDept() + ",105";
            List<String> managerList = new ArrayList<>();
            managerList.add("WangRongHua");
            managerList.add("YuSiHan");
            String[] deptId = deptString.split(",");
            for (String dept : deptId) {
                    List<SysUser> manageList = getManageId(dept, "");
                    for (SysUser manager : manageList) {
                        managerList.add(manager.getPinyin());

                }
            }
            MessageController.sendAppMessage(message, StringUtils.join(managerList, "|"));
        }
        return success();
    }

    /**
     * 删除cmc项目
     */
    @Log(title = "cmc项目", businessType = BusinessType.DELETE)
	@DeleteMapping("/{projectIds}")
    public AjaxResult remove(@PathVariable String[] projectIds)
    {
        cmcProjectWorkService.deleteCmcProjectWorkByProjectIds(projectIds);
        return success(cmcProjectService.deleteCmcProjectByProjectIds(projectIds));
    }

    public String deptIdToName(CmcProject cmcProject) {
        String underTakingDept = cmcProject.getUndertakingDept();
        StringBuilder underTakingDeptName = new StringBuilder();
        if (underTakingDept != null && !underTakingDept.equals("") && underTakingDept.split(",").length > 0) {
            for (String dept : underTakingDept.split(","))
                underTakingDeptName.append(deptService.selectDeptById(Long.parseLong(dept)).getDeptName()).append(",");
            cmcProject.setUndertakingDeptName(underTakingDeptName.substring(0, underTakingDeptName.length() - 1));
        }
        return cmcProject.getUndertakingDeptName();
    }

    public String inspectorIdToName(CmcProject cmcProject) {
        String qualityInspector = cmcProject.getQualityInspector();
        StringBuilder qualityInspectorName = new StringBuilder();
        if (qualityInspector != null && !qualityInspector.equals("") && qualityInspector.split(",").length > 0) {
            for (String inspector : qualityInspector.split(","))
                qualityInspectorName.append(userService.selectUserById(Long.parseLong(inspector)).getNickName()).append(",");
            cmcProject.setQualityInspectorName(qualityInspectorName.substring(0, qualityInspectorName.length() - 1));
        }
        return cmcProject.getQualityInspectorName();
    }

    public List<SysUser> getManageId(String deptId, String partyAId) {
        List<SysUser> userList = new ArrayList<>();
        CmcManageDept cmcManageDept = new CmcManageDept();
        cmcManageDept.setDeptId(deptId);
        cmcManageDept.setPartyAId(partyAId);
        for (CmcManageDept manageDept : cmcManageDeptService.selectCmcManageDeptList(cmcManageDept)) {
            userList.add(userService.selectUserById(manageDept.getUserId()));
        }
        return userList;
    }

    //院内外项目数量
    public void getProjectSourceStatistic(CmcProject cmcProject, JSONObject sourceObject) {
        cmcProject.setProjectSource("1");
        sourceObject.put("院外", cmcProjectService.selectCmcAllProjectList(cmcProject).size());
        cmcProject.setProjectSource("0");
        sourceObject.put("院内", cmcProjectService.selectCmcAllProjectList(cmcProject).size());
        cmcProject.setProjectSource(null);
    }

    //各类型项目数量
    public void getProjectTypeStatistic(CmcProject cmcProject, JSONObject typeObject) {
        //类型太多，循环效率不高-直接用Java特性进行分组统计
        Map<Object, Long> typeGroup = cmcProjectService.selectCmcAllProjectList(cmcProject).stream().collect(Collectors.groupingBy(item -> Optional.ofNullable(item.getProjectType()), Collectors.counting()));
        List<Map.Entry<Object, Long>> typeList = new ArrayList<>(typeGroup.entrySet());
        typeList.sort(Map.Entry.comparingByValue());
        typeList.forEach((item) -> {
            if (item.getKey() != Optional.empty())
                typeObject.put(item.toString().substring(item.toString().indexOf("[") + 1, item.toString().indexOf("]")), item.getValue());
        });
    }

    //各部门承担项目数量
    public void getProjectDeptStatistic(CmcProject cmcProject, JSONObject deptObject) {
        for (int i = 107; i <= 115; i++) {
            cmcProject.setUndertakingDept(String.valueOf(i));
            List<CmcProject> deptList = new ArrayList<>();
            if (i != 113)
                deptList = cmcProjectService.selectCmcProjectList(cmcProject);
            else
                deptList = cmcProjectService.selectCmcInvestProjectList(cmcProject);
            deptObject.put(deptService.selectDeptById((long)i).getDeptName(), deptList.size());
        }
        cmcProject.setUndertakingDept(null);
    }

}
