package com.ruoyi.web.controller.oa;

import java.math.BigDecimal;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.List;
import javax.servlet.http.HttpServletResponse;

import com.alibaba.fastjson2.JSONArray;
import com.alibaba.fastjson2.JSONObject;
import com.ruoyi.common.utils.SnowFlake;
import com.ruoyi.oa.domain.CmcSubContract;
import com.ruoyi.oa.domain.CmcSubContract;
import com.ruoyi.oa.domain.CmcPartner;
import com.ruoyi.oa.service.ICmcPartnerService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.oa.domain.CmcSubContract;
import com.ruoyi.oa.service.ICmcSubContractService;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.common.core.page.TableDataInfo;

/**
 * cmc分包合同评审Controller
 * 
 * @author cmc
 * @date 2024-04-18
 */
@RestController
@RequestMapping("/oa/subContract")
public class CmcSubContractController extends BaseController
{
    @Autowired
    private ICmcSubContractService cmcSubContractService;

    /**
     * 查询cmc分包合同评审列表
     */
    @GetMapping("/list")
    public TableDataInfo list(CmcSubContract cmcSubContract)
    {
        startPage();
        List<CmcSubContract> list = cmcSubContractService.selectCmcSubContractList(cmcSubContract);
        return getDataTable(list);
    }

    /**
     * 导出cmc分包合同评审列表
     */
    @Log(title = "cmc分包合同评审", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, CmcSubContract cmcSubContract)
    {
        List<CmcSubContract> list = cmcSubContractService.selectCmcSubContractList(cmcSubContract);
        ExcelUtil<CmcSubContract> util = new ExcelUtil<CmcSubContract>(CmcSubContract.class);
        util.exportExcel(response, list, "cmc分包合同评审数据");
    }

    /**
     * 获取cmc分包合同评审详细信息
     */
    @GetMapping(value = "/{subContractId}")
    public AjaxResult getInfo(@PathVariable("subContractId") String subContractId)
    {
        return success(cmcSubContractService.selectCmcSubContractBySubContractId(subContractId));
    }

    /**
     * 获取cmc分包合同统计信息
     */
    @GetMapping("/statistic")
    public AjaxResult getSubContractStatistic(CmcSubContract cmcSubContract) throws ParseException {
        JSONObject jsonObject = new JSONObject();
        JSONArray yearArray = new JSONArray();
        JSONObject yearObject = new JSONObject();
        JSONArray sourceArray = new JSONArray();
        JSONObject sourceObject = new JSONObject();
        JSONArray amountArray = new JSONArray();
        JSONObject amountObject = new JSONObject();
        JSONArray cwAmountArray = new JSONArray();
        JSONObject cwAmountObject = new JSONObject();
        //每年合同数量及金额
        if (cmcSubContract.getSignDate() == null) {
            for (int i = 2019; i <= Calendar.getInstance().get(Calendar.YEAR); i++) {
                cmcSubContract.setSignDate(new SimpleDateFormat("yyyy").parse(String.valueOf(i)));
                List<CmcSubContract> yearList = cmcSubContractService.selectCmcSubContractList(cmcSubContract);
                yearObject.put(String.valueOf(i), yearList.size());
                BigDecimal amount = new BigDecimal(0);
                for (CmcSubContract subContract : yearList) {
                    amount = amount.add(subContract.getSubAmount());
                }
                amountObject.put(String.valueOf(i), amount);
            }
            cmcSubContract.setSignDate(null);
        }
        else {
            //输入年份合同数量及金额
            yearObject.put(new SimpleDateFormat("yyyy").format(cmcSubContract.getSignDate()), cmcSubContractService.selectCmcSubContractList(cmcSubContract).size());
            BigDecimal amount = new BigDecimal(0);
            for (CmcSubContract subContract : cmcSubContractService.selectCmcSubContractList(cmcSubContract)) {
                amount = amount.add(subContract.getSubAmount());
            }
            amountObject.put(new SimpleDateFormat("yyyy").format(cmcSubContract.getSignDate()), amount);
        }
        getProjectSourceStatistic(cmcSubContract, sourceObject, cwAmountObject);
        yearArray.add(yearObject);
        sourceArray.add(sourceObject);
        amountArray.add(amountObject);
        cwAmountArray.add(cwAmountObject);
        jsonObject.put("year", yearArray);
        jsonObject.put("source", sourceArray);
        jsonObject.put("amount", amountArray);
        jsonObject.put("cwAmount", cwAmountArray);
        return success(jsonObject);
    }

    /**
     * 新增cmc分包合同评审
     */
    @Log(title = "cmc分包合同评审", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody CmcSubContract cmcSubContract)
    {
        return toAjax(cmcSubContractService.insertCmcSubContract(cmcSubContract));
    }

    /**
     * 修改cmc分包合同评审
     */
    @Log(title = "cmc分包合同评审", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody CmcSubContract cmcSubContract)
    {
        if (cmcSubContract.getManagerComment() != null && cmcSubContract.getGmComment() == null) {
            cmcSubContract.setManagerUserId(getLoginUser().getUserId());
        }
        if (cmcSubContract.getGmComment() != null && cmcSubContract.getSignDate() == null) {
            cmcSubContract.setGmUserId(getLoginUser().getUserId());
        }
        return toAjax(cmcSubContractService.updateCmcSubContract(cmcSubContract));
    }

    /**
     * 删除cmc分包合同评审
     */
    @Log(title = "cmc分包合同评审", businessType = BusinessType.DELETE)
	@DeleteMapping("/{subContractIds}")
    public AjaxResult remove(@PathVariable String[] subContractIds)
    {
        return success(cmcSubContractService.deleteCmcSubContractBySubContractIds(subContractIds));
    }

    //院内外项目数量及金额
    public void getProjectSourceStatistic(CmcSubContract cmcSubContract, JSONObject sourceObject, JSONObject cwAmountObject) {
        cmcSubContract.setProjectSource("1");
        List<CmcSubContract> sourceList = cmcSubContractService.selectCmcSubContractList(cmcSubContract);
        BigDecimal cAmount = new BigDecimal(0);
        BigDecimal wAmount = new BigDecimal(0);
        for (CmcSubContract subContract : sourceList) {
            wAmount = wAmount.add(subContract.getSubAmount());
        }
        sourceObject.put("院外", sourceList.size());
        cwAmountObject.put("院外", wAmount);
        cmcSubContract.setProjectSource("0");
        sourceList = cmcSubContractService.selectCmcSubContractList(cmcSubContract);
        for (CmcSubContract subContract : sourceList) {
            cAmount = cAmount.add(subContract.getSubAmount());
        }
        sourceObject.put("院内", sourceList.size());
        cwAmountObject.put("院内", cAmount);
        cmcSubContract.setProjectSource(null);
    }

}
